/*
 * Copyright 2015-2019 ISP RAS (http://www.ispras.ru)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.testbase.knowledge.branch.beq;

import ru.ispras.fortress.randomizer.Randomizer;
import ru.ispras.testbase.knowledge.branch.IfThenElseGenerator;
import ru.ispras.testbase.knowledge.integer.IntNumber;

/**
 * {@link BeqThenElseGenerator} is a test data generator for BEQ instructions.
 *
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public final class BeqThenElseGenerator extends IfThenElseGenerator {
  public static boolean checkOperandsThen(final IntNumber[] operands) {
    return operands[0].getValue(true) == operands[1].getValue(true);
  }

  public static IntNumber[] generateOperandsThen(final IntNumber[] operands) {
    // The first operand is always unknown because it is chosen to be used as a stream register.
    final long value = operands[1] != null
        ? operands[1].getValue(true)
        : Randomizer.get().nextLongRange(MIN_VALUE, MAX_VALUE);
    
    return new IntNumber[] {
        new IntNumber(FORMAT, value),
        new IntNumber(FORMAT, value)
    };
  }

  public static IntNumber[] generateOperandsElse(final IntNumber[] operands) {
    // The first operand is always unknown because it is chosen to be used as a stream register.
    final long value = operands[1] != null
        ? operands[1].getValue(true)
        : Randomizer.get().nextLongRange(MIN_VALUE, MAX_VALUE);
    
    return new IntNumber[] {
        new IntNumber(FORMAT, distinctValue(value)),
        new IntNumber(FORMAT, value)
    };
  }

  public static long distinctValue(final long value) {
    long result;
    do {
      result = Randomizer.get().nextLongRange(MIN_VALUE, MAX_VALUE);
    } while (result == value);
    return result;
  }

  @Override
  public boolean checkThen(final IntNumber[] operands) {
    return checkOperandsThen(operands);
  }
  
  @Override
  public IntNumber[] generateThen(final IntNumber[] operands) {
    return generateOperandsThen(operands);
  }

  @Override
  public IntNumber[] generateElse(final IntNumber[] operands) {
    return generateOperandsElse(operands);
  }
}
