/*
 * Copyright 2008-2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.testbase.knowledge.iterator;

/**
 * {@link Iterator} is a generic iterator interface.
 *
 * @param <T> the item type.
 *
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public interface Iterator<T> {
  /**
   * Initializes the iterator.
   */
  void init();

  /**
   * Checks if the iterator is not exhausted (a value is available).
   *
   * @return {@code true} if the iterator is not exhausted; {@code false} otherwise.
   */
  boolean hasValue();

  /**
   * Returns the current value of the iterator.
   *
   * @return the current value of the iterator.
   */
  T value();

  /**
   * Makes an iteration.
   */
  void next();

  /**
   * Stops the iterator.
   */
  void stop();

  /**
   * Creates a copy of the iterator.
   *
   * @return A copy of the iterator.
   */
  Iterator<T> clone();
}
