/*
 * Copyright 2014 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.testbase.knowledge.branch;

import org.junit.Assert;
import org.junit.Test;
import ru.ispras.testbase.knowledge.basis.GeneratorResult;
import ru.ispras.testbase.knowledge.branch.beq.BeqThenElseGenerator;
import ru.ispras.testbase.knowledge.branch.beqz.BeqzThenElseGenerator;
import ru.ispras.testbase.knowledge.branch.bge.BgeThenElseGenerator;
import ru.ispras.testbase.knowledge.branch.bgeu.BgeuThenElseGenerator;
import ru.ispras.testbase.knowledge.branch.bgez.BgezThenElseGenerator;
import ru.ispras.testbase.knowledge.branch.bgt.BgtThenElseGenerator;
import ru.ispras.testbase.knowledge.branch.bgtu.BgtuThenElseGenerator;
import ru.ispras.testbase.knowledge.branch.bgtz.BgtzThenElseGenerator;
import ru.ispras.testbase.knowledge.branch.ble.BleThenElseGenerator;
import ru.ispras.testbase.knowledge.branch.bleu.BleuThenElseGenerator;
import ru.ispras.testbase.knowledge.branch.blez.BlezThenElseGenerator;
import ru.ispras.testbase.knowledge.branch.blt.BltThenElseGenerator;
import ru.ispras.testbase.knowledge.branch.bltu.BltuThenElseGenerator;
import ru.ispras.testbase.knowledge.branch.bltz.BltzThenElseGenerator;
import ru.ispras.testbase.knowledge.branch.bne.BneThenElseGenerator;
import ru.ispras.testbase.knowledge.branch.bnez.BnezThenElseGenerator;
import ru.ispras.testbase.knowledge.integer.IntNumber;

/**
 * Unit tests for {@link IfThenElseGenerator} subclasses.
 *
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public final class IfThenElseGeneratorTestCase {
  public static final int TEST_NUMBER = 1000;
  
  private void runTest(final IfThenElseGenerator generator) {
    for (int i = 0; i < TEST_NUMBER; i++) {
      final IfThenElse situation = (i & 1) == 0 ? IfThenElse.THEN : IfThenElse.ELSE;

      final GeneratorResult<IntNumber> result = generator.generate(situation, new IntNumber[2]);
      Assert.assertTrue(result.operands.length == 1 || result.operands.length == 2);

/*      if (result.operands.length == 1) {
        System.out.format("%s %x\n", situation.name(),
            result.operands[0].getValue(true));
      } else {
        System.out.format("%s %x %x\n", situation.name(),
            result.operands[0].getValue(true), result.operands[1].getValue(true));
      }*/
      Assert.assertTrue(result.status);
    }
  }
  
  @Test
  public void runTestBeq() {
    runTest(new BeqThenElseGenerator());
  }
  
  @Test
  public void runTestBeqz() {
    runTest(new BeqzThenElseGenerator());
  }
  
  @Test
  public void runTestBge() {
    runTest(new BgeThenElseGenerator());
  }
  
  @Test
  public void runTestBgeu() {
    runTest(new BgeuThenElseGenerator());
  }
  
  @Test
  public void runTestBgez() {
    runTest(new BgezThenElseGenerator());
  }
  
  @Test
  public void runTestBgt() {
    runTest(new BgtThenElseGenerator());
  }
  
  @Test
  public void runTestBgtu() {
    runTest(new BgtuThenElseGenerator());
  }
  
  @Test
  public void runTestBgtz() {
    runTest(new BgtzThenElseGenerator());
  }
  
  @Test
  public void runTestBle() {
    runTest(new BleThenElseGenerator());
  }
  
  @Test
  public void runTestBleu() {
    runTest(new BleuThenElseGenerator());
  }
  
  @Test
  public void runTestBlez() {
    runTest(new BlezThenElseGenerator());
  }
  
  @Test
  public void runTestBlt() {
    runTest(new BltThenElseGenerator());
  }
  
  @Test
  public void runTestBltu() {
    runTest(new BltuThenElseGenerator());
  }
  
  @Test
  public void runTestBltz() {
    runTest(new BltzThenElseGenerator());
  }

  @Test
  public void runTestBne() {
    runTest(new BneThenElseGenerator());
  }
  
  @Test
  public void runTestBnez() {
    runTest(new BnezThenElseGenerator());
  }
}
